# test_foo.py IS a pytest test, as well as Python.
# This test doesn't test anything meaningful. It's just here for filetype tests.
# But it should pass.

def test_reality():
    code = 1
    assert code == 1
